<?php

/**
 * @copyright Copyright © 2013-2025 Codemarket - Todos os direitos reservados
 * @author    Felipo Antonoff <contato@codemarket.com.br>
 * @link      https://www.codemarket.com.br
 * @link      https://felipoantonoff.com
 * @date      16/04/2025 12:30
 **/

namespace Codemarket\System\Library\CodeApp;

/**
 * © Copyright 2013-2025 Codemarket - Todos os direitos reservados.
 * Usado para métodos auxíliares ligados ao Banco de Dados, Cache e outros.
 *
 * @package CodeApp
 */
class HelperDB
{
    private $config;
    private $session;
    private $cache;
    private $db;
    private $log;

    /**
     * Construtor que chama o setup().
     *
     * @param $registry
     */
    public function __construct($registry)
    {
        $this->config = $registry->get('config');
        $this->session = $registry->get('session');
        $this->cache = $registry->get('cache');
        $this->db = $registry->get('db');

        /*
        $this->load->model('module/codemarket_module');
        $conf = $this->model_module_codemarket_module->getModulo('');
        */
        $this->log = new \log('Code-HelperDB-' . date('d-m-Y') . '.log');
    }

    /**
     * Apenas teste o Helper com uma mensagem.
     *
     * @return void
     */
    public function test()
    {
        exit("HelperDB OK");
    }

    /**
     * Retorna a lista de Status de Pedidos na forma de String separado por vírgula.
     *
     * @return string
     */
    public function listOrderStatus(): string
    {
        $query = $this->db->query("SELECT DISTINCT(order_status_id) FROM " . DB_PREFIX . "order_status")->rows;

        if (empty($query)) {
            return '';
        }

        $order_status_ids = '';

        foreach ($query as $order_status) {
            $order_status_ids .= $order_status['order_status_id'] . ',';
        }

        $order_status_ids = trim($order_status_ids, ',');
        $this->log->write('listOrderStatus() - Retornado a lista de Order Status: ' . $order_status_ids);

        return $order_status_ids;
    }

    /**
     * curl
     * Método Auxiliar para uso da API
     * Envia o Curl usando GET, POST, PUT ou outro método.
     *
     * @param string      $url             URL
     * @param string      $method          Método
     * @param string      $function_name   Nome da função
     * @param array       $data            Dados
     * @param array       $data_header     Dados do Header
     * @param string      $token           Token para usar na Autentificação
     * @param int         $timeout         Timeout
     * @param int         $debug           Se 0 desativado, se 1 salva nos Logs e 2 exibe os dados
     * @param object|null $log             Passa o sistema de Log
     * @param bool        $response_header Retorna os dados do Header do Response
     *
     * @return false|mixed
     */
    public function curl(
        string $url,
        string $method = "POST",
        string $function_name = '',
        array $data = [],
        array $data_header = [],
        string $token = '',
        int $timeout = 8,
        int $debug = 0,
        object $log = null,
        bool $response_header = false
    ) {
        if (empty($url)) {
            return false;
        }

        // Mudando o sistema de Log
        if ($log) {
            $this->log = $log;
        }

        if (empty($data_header)) {
            if (!empty($token)) {
                $data_header = [
                    'content-type: application/json',
                    'accept: application/json',
                    'Cache-Control: no-cache',
                    // https://developer.mozilla.org/pt-BR/docs/Web/HTTP/Reference/Headers/Content-Encoding
                    'Accept-Encoding: gzip, deflate', // Retirado o br, pois é o Brotli e tem hospedagem sem suporte
                    'Authorization: Bearer ' . $token,
                ];
            } else {
                $data_header = [
                    'content-type: application/json',
                    'accept: application/json',
                    'Cache-Control: no-cache',
                    'Accept-Encoding: gzip, deflate',
                ];
            }
            //print_r($data_header); exit();
        }

        $curl = curl_init();

        $dataPost = !empty($data) ? $data : [];

        if ($response_header) {
            $curl_option = [
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_HEADER => false,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 3,
                CURLOPT_TIMEOUT => $timeout,
                CURLOPT_CONNECTTIMEOUT => 2,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => $method,
                CURLOPT_POSTFIELDS => !empty($dataPost) ? json_encode($dataPost)
                    : json_encode([]),
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_SSL_VERIFYHOST => 2,
                CURLOPT_SSL_VERIFYPEER => 1,
                CURLOPT_HTTPHEADER => $data_header,
                CURLOPT_SSLCERTPASSWD => "",
                CURLOPT_HEADERFUNCTION => function ($curl, $header) use (&$responseHeaders) {
                    $len = strlen($header);
                    $parts = explode(':', $header, 2);

                    if (count($parts) === 2) {
                        $responseHeaders[strtolower(trim($parts[0]))] = trim($parts[1]);
                    }

                    return $len;
                },
            ];
        } else {
            // Identico ao acima, só que sem o CURLOPT_HEADERFUNCTION
            $curl_option = [
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_HEADER => false,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 3,
                CURLOPT_TIMEOUT => $timeout,
                CURLOPT_CONNECTTIMEOUT => 2,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => $method,
                CURLOPT_POSTFIELDS => !empty($dataPost) ? json_encode($dataPost)
                    : json_encode([]),
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_SSL_VERIFYHOST => 2,
                CURLOPT_SSL_VERIFYPEER => 1,
                CURLOPT_HTTPHEADER => $data_header,
                CURLOPT_SSLCERTPASSWD => ""
            ];
        }

        curl_setopt_array($curl, $curl_option);

        $response = curl_exec($curl);
        $error = curl_error($curl);
        $info = curl_getinfo($curl);
        curl_close($curl);

        //print_r($curl_option);
        //print_r($response);
        //exit();

        // Modo Debug ativo para o retorno do Curl
        if ($debug === 2) {
            echo "<h1>URL: " . $url . ", Função: " . $function_name . ", Método: " . $method . "</h1>";

            if ($response_header === false) {
                echo "<h2>Dados Curl</h2>";
                print_r($curl_option);
            }

            echo "<h2>Dados da Requisição</h2>";
            print_r($data);

            echo "<h2>Header da Requisição</h2>";
            print_r($data_header);

            if (!empty($responseHeaders)) {
                echo "<h2>Header Resposta</h2>";
                print_r($responseHeaders);
            }

            echo "<h2>Resposta</h2>";
            print_r($response);

            echo "<h2>Erro</h2>";
            print_r($error);

            echo "<h2>Info do Curl</h2>";
            print_r($info);

            exit();
        }

        // Verificando se teve erro ou retorno vazio
        if (($error || empty($response)) && ($info['http_code'] >= 400)) {
            if ($debug === 1) {
                $this->log->write(
                    'curl() ' . $function_name . ' - Error Curl, URL: ' . $url
                    . ', HTTP Code: ' . $info['http_code'] . ', Erro: '
                    . print_r($error, true) . ', Retorno: ' . print_r(
                        $response,
                        true
                    ) . ', Dados: '
                    . print_r($dataPost, true)
                );
            }

            return false;
        }

        if ($debug === 1) {
            $this->log->write(
                'curl() ' . $function_name . ' - Rodado com sucesso, URL: ' . $url
                . ', HTTP Code: ' . $info['http_code'] . ', Retorno: '
                . print_r($response, true) . ', Dados: '
                . print_r($dataPost, true)
            );
        }

        if (empty($response)) {
            return [];
        }
        //$response = json_decode($response, true);
        $response = codeHelperJsonDecode($response, true, false, true);

        // Retornando os dados do Header do Response
        if ($response_header && isset($responseHeaders)) {
            $response2['response'] = $response;
            $response2['header'] = $responseHeaders;

            return $response2;
        }

        return $response;
    }

    /**
     *  curlMulti
     *  Método Auxiliar para uso da API
     *  Envia o Curl por POST para a lista de requisições enviadas, fazendo mais de uma chamada ao mesmo tempo
     *  Faz uso do Curl Multi https://www.php.net/manual/pt_BR/function.curl-multi-init.php.
     *
     * @param array       $requests    Array simples das requisições a serem feitas, exemplo: [1,2,3]
     * @param string      $url         URL
     * @param array       $dataHeader  Dados do Header
     * @param int         $timeout     Timeout
     * @param int         $timeoutAll  Timeout geral da função
     * @param int         $maxRequests Quantidade máxima de chamadas Simultâneas
     * @param object|null $log         Passa o sistema de Log
     * @param bool        $return      Retorna os dados do Response
     *
     * @return array|false|string|void
     */
    public function curlMulti(
        array $requests,
        string $url,
        array $dataHeader = [],
        int $timeout = 30,
        int $timeoutAll = 30,
        int $maxRequests = 5,
        object $log = null,
        bool $return = true
    ) {
        if (empty($requests) || empty($url)) {
            return false;
        }

        if ($return) {
            // Inicia o buffer de saída
            ob_start();
        }

        // Mudando o sistema de Log
        if ($log) {
            $this->log = $log;
        }

        $startTime = time();  // Marca o início do processo

        $mh = curl_multi_init();  // Inicializa o multi-handle do Curl
        //$maxRequests = 20;  // Limite de requisições simultâneas
        //$timeoutAll = 60;  // Timeout global de 60 segundos
        $handles = [];  // Armazena os handles do Curl associados aos IDs dos requests

        //$dataHeader = ['Content-Type: application/json'];
        $method = 'POST';
        $response = [];

        if (empty($dataHeader)) {
            // https://en.wikipedia.org/wiki/List_of_HTTP_header_fields
            $dataHeader = [
                'content-type: application/json',
                'accept: application/json',
                'Cache-Control: no-cache',
                'Accept-Encoding: gzip, deflate',
            ];
        }

        // Evitando Request com valor repetido
        $requests = array_unique($requests);

        // Limitando cada rodada com base no maxRequests
        $requests = array_chunk($requests, $maxRequests);
        //print_r($requests);

        /*
         * Array
            (
                [0] => Array
                    (
                        [0] => 10092941
                        [1] => 10093783
                    )

                [1] => Array
                    (
                        [0] => 10088293
                    )

            )
        */
        foreach ($requests as $batchIndex => $batch) {
            echo "<h2>Processando lote " . ($batchIndex + 1) . " de " . count($batch) . " requisições</h2>";

            // Cria os handles para o lote atual
            foreach ($batch as $requestID) {
                $curlOptions = [
                    CURLOPT_URL => $url,
                    CURLOPT_RETURNTRANSFER => true,
                    CURLOPT_HEADER => false,
                    CURLOPT_ENCODING => '',
                    CURLOPT_MAXREDIRS => 3,
                    CURLOPT_TIMEOUT => $timeout,  // Timeout por requisição
                    CURLOPT_CONNECTTIMEOUT => 2,
                    CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                    CURLOPT_CUSTOMREQUEST => $method,
                    CURLOPT_POSTFIELDS => ['requestID' => $requestID],
                    CURLOPT_FOLLOWLOCATION => true,
                    CURLOPT_SSL_VERIFYHOST => 2,
                    CURLOPT_SSL_VERIFYPEER => 1,
                    CURLOPT_HTTPHEADER => $dataHeader,
                    CURLOPT_SSLCERTPASSWD => "",
                ];

                $curl = curl_init();
                curl_setopt_array($curl, $curlOptions);
                curl_multi_add_handle($mh, $curl);  // Adiciona o handle ao multi-handle
                $handles[$requestID] = $curl;  // Associa o handle ao ID do pedido
            }

            // Executa as requisições do lote atual
            do {
                $status = curl_multi_exec($mh, $running);
                curl_multi_select($mh);

                while ($info = curl_multi_info_read($mh)) {
                    $curl = $info['handle'];
                    $requestID = array_search($curl, $handles);

                    if ($requestID === false) {
                        continue;
                    }

                    if (curl_errno($curl)) {
                        if ($return) {
                            $response[$requestID] = "<h3>Requisição $requestID falhou com erro: " . curl_error($curl) . "</h3>";
                        } else {
                            echo "<h3>Requisição $requestID falhou com erro: " . curl_error($curl) . "</h3>";
                        }
                    } else {
                        if ($return) {
                            $response[$requestID] = curl_multi_getcontent($curl);
                        } else {
                            $response = curl_multi_getcontent($curl);
                            echo $response;
                        }
                    }

                    curl_multi_remove_handle($mh, $curl);
                    curl_close($curl);
                    unset($handles[$requestID]);
                }

                if (time() - $startTime >= $timeoutAll) {
                    foreach ($handles as $pendingRequestID => $pendingHandle) {
                        echo "<h3>Requisição $pendingRequestID não processada</h3>";
                    }

                    curl_multi_close($mh);

                    if ($return) {
                        // Qualquer saída anterior será capturada no buffer e não enviada ao cliente
                        ob_end_clean(); // Limpa o buffer e descarta qualquer saída

                        return $response;
                    }

                    exit("<h2>Timeout global de $timeoutAll segundos atingido</h2>");
                }
            } while ($running > 0);

            $handles = [];
        }

        curl_multi_close($mh);

        if ($return) {
            // Qualquer saída anterior será capturada no buffer e não enviada ao cliente
            ob_end_clean(); // Limpa o buffer e descarta qualquer saída

            return $response;
        }
        echo "<h2>Todos os lotes foram processados com sucesso</h2>";
    }

    /**
     * Cria um Semáforo usando o Redis se habilitado.
     *
     * @param string $key     Chave do Semáforo
     * @param int    $timeout Tempo em segundos do Semáforo
     *
     * @throws \Exception
     * @return array|true
     */
    public function SemaphoreAcquire(string $key, int $timeout)
    {
        if (!class_exists('Redis')) {
            return true;
        }

        $cache = new \Cache('redis', $timeout);

        $lockTime = $cache->get($key);

        // Garante que o valor vindo do cache seja tratado como float (mesmo se vier como string)
        $lockTime = ($lockTime !== false) ? (float) $lockTime : false;

        $currentTime = microtime(true); // Timestamp atual

        // Verifica se há bloqueio expirado
        if ($lockTime !== false && ($currentTime - $lockTime) > $timeout) {
            $cache->delete($key);
            $lockTime = false; // Considera como se o lock nunca existiu
        }

        // Lock ainda ativo?
        if ($lockTime !== false) {
            $expirationTime = $lockTime + $timeout;

            return [
                'status' => false,
                'date' => date('d/m/Y H:i:s', (int) $lockTime),
                'expire' => date('d/m/Y H:i:s', (int) floor($expirationTime))
            ];
        }

        // Tenta criar o lock
        $set = $cache->set($key, $currentTime);
        $expirationTime = $currentTime + $timeout;

        return [
            'status' => $set,
            'date' => date('d/m/Y H:i:s', (int) $currentTime),
            'expire' => date('d/m/Y H:i:s', (int) floor($expirationTime))
        ];
    }

    /**
     * Aguarda o Semáforo ser liberado ou até o tempo limite.
     *
     * @param string $key          Chave do Semáforo
     * @param int    $timeout      Tempo em segundos do Semáforo
     * @param int    $timeoutAwait Tempo limite em Segundos para aguardar o Semáforo liberar
     * @param float  $pause        Tempo de pausa em Segundos na verificação do Semáforo
     * @param bool   $debug        Se 1 vai exibir as mensagens
     *
     * @return bool
     */
    public function SemaphoreAcquireAwait(
        string $key,
        int $timeout = 10,
        int $timeoutAwait = 10,
        float $pause = 0.5,
        bool $debug = false
    ): bool {
        $start_time = microtime(true);
        $debug_msg = "<h1>Semáforo Debug</h1>";
        $debug_msg .= "<h2>Verificando Semáforo Key: " . $key . ", Timeout: " . $timeout . ", Timeout Await: " . $timeoutAwait . ", Pausa: " . $pause . " segundos " . codeHelperTimeNow() . "</h2>";

        // Aguarda até chegar no tempo limite ou até o Semáforo ser liberado
        while (true) {
            //while ((microtime(true) - $start_time) < $timeoutAwait) {
            $elapsed_time = microtime(true) - $start_time; // Tempo já passado

            // Se o tempo de espera exceder o limite, interrompe
            if ($elapsed_time > $timeoutAwait) {
                break;
            }

            $acquire = $this->SemaphoreAcquire($key, $timeout);

            if (!empty($acquire['status'])) {
                $debug_msg .= "<h2>Semáforo Key: " . $key . " Livre (<b style='color:green;'>VERDE</b>), Expira em: "
                    . $acquire['expire'] . " - " .
                    codeHelperTimeNow() . "</h2>";

                if ($debug) {
                    echo $debug_msg;
                }

                return true;
                // Semáforo liberado
            }

            if (!empty($acquire['expire'])) {
                $debug_msg .= "<h2>Semáforo Key: " . $key . " Fechado (<b style='color:red;'>VERMELHO</b>), Expira em: "
                    . $acquire['expire'] . " - " .
                    codeHelperTimeNow() . "</h2>";
            } else {
                $debug_msg .= "<h2>Semáforo Key: " . $key . " Fechado (<b style='color:red;'>VERMELHO</b>) " .
                    codeHelperTimeNow() . "</h2>";
            }

            // Sem pausa
            if (empty($pause) || $pause <= 0) {
                return false;
            }

            // Pausa até a próxima verificação em intervalo de segundos
            usleep((int) ($pause * 1000000)); // usleep espera em microssegundos (1 segundo = 1.000.000 microssegundos)
        }

        if ($debug) {
            echo $debug_msg . "<h2>Semáforo Key: " . $key . " Fechado (<b style='color:red;'>VERMELHO</b>) " . codeHelperTimeNow() . "</h2>";
        }

        // Semáforo não liberado e chegou no tempo limite
        return false;
    }
}
