<?php

/**
 * @copyright Copyright © 2013-2025 Codemarket - Todos os direitos reservados
 * @author    Felipo Antonoff <contato@codemarket.com.br>
 * @link      https://www.codemarket.com.br
 * @link      https://felipoantonoff.com
 * @date      16/04/2025 17:02
 **/

/*
 * Contém funções auxiliares para ser usado nos Apps da Codemarket
 *
 * @package CodeApp
 */

/**
 * Cria o UUID V4
 * Um Identificador Único Universal contendo até 128 bits
 * Função semelhante ao https://www.php.net/manual/pt_BR/function.uniqid.php.
 *
 * @return string
 */
function codeHelperUuidv4(): string
{
    $uuid = random_bytes(16);

    // Define a versão para 4 (UUID v4)
    $uuid[6] = chr((ord($uuid[6]) & 0x0f) | 0x40);

    // Define a variante para 10xx (padrão RFC 4122)
    $uuid[8] = chr((ord($uuid[8]) & 0x3f) | 0x80);

    // Converte os bytes para hexadecimal
    $hex = bin2hex($uuid);

    return sprintf(
        '%s-%s-%s-%s-%s',
        substr($hex, 0, 8),
        substr($hex, 8, 4),
        substr($hex, 12, 4),
        substr($hex, 16, 4),
        substr($hex, 20, 12)
    );
}

/**
 * Cria um Semáforo e retorna o seu ID se liberado dentro do tempo limite.
 *
 * @param int   $key         Chave do Semáforo
 * @param int   $max_acquire Máximo de Ocorrências
 * @param int   $timeout     Tempo limite em Segundos para aguardar o Semáforo liberar
 * @param float $pause       Tempo de pausa em Segundos na verificação do Semáforo
 * @param bool  $debug       Se 1 vai exibir as mensagens
 */
function codeHelperSemaphore(
    int $key = 1,
    int $max_acquire = 1,
    int $timeout = 10,
    float $pause = 0.5,
    bool $debug =
    false
) {
    if (!extension_loaded('sysvsem')) {
        // Sem a extensão
        return true;
    }

    //$key = ftok(__FILE__, 'a'); ou ftok(__FILE__, 'o')
    $semaphore_id = sem_get($key, $max_acquire);

    $start_time = time();
    $debug_msg = "<h1>Semáforo Debug</h1>";
    $debug_msg .= "<h2>Verificando Semáforo " . date('d/m/Y H:i:s') . "</h2>";

    // Aguarda até chegar no tempo limite ou até o Semáforo ser liberado
    while ((time() - $start_time) < $timeout) {
        if (sem_acquire($semaphore_id, true)) {
            $debug_msg .= "<h2>Semáforo " . $key . " Livre (<b style='color:green;'>VERDE</b>) ID " . $semaphore_id . " " . date(
                'd/m/Y H:i:s'
            ) . "</h2>";

            if ($debug) {
                echo $debug_msg;
            }

            return $semaphore_id;
            // Semáforo liberado
            //return ['status' => true, 'semaphore_id' => $semaphore_id];
        }
        $debug_msg .= "<h2>Semáforo " . $key . " Fechado (<b style='color:red;'>VERMELHO</b>) ID " . $semaphore_id . " " . date(
            'd/m/Y H:i:s'
        ) . "</h2>";

        // Sem pausa
        if (empty($pause)) {
            return false;
        }

        // Pausa até a próxima verificação
        usleep(($pause * 1000000));
    }

    if ($debug) {
        echo $debug_msg . "<h2>Semáforo " . $key . " Fechado (<b style='color:red;'>VERMELHO</b>) ID " . $semaphore_id . " " . date(
            'd/m/Y H:i:s'
        ) . "</h2>";
    }

    // Semáforo não liberado e chegou no tempo limite
    return false;
    //return ['status' => false];

    /*
    try {
        // CÓDIGO PARA RODAR AQUI
    } catch (Exception $e) {
        echo "Erro: " . $e->getMessage() . "\n";
    } finally {
        // Liberando o Semáforo
        sem_release($semaphore_id);
    }
    */
}

/**
 * Libera o Semáforo.
 *
 * @param $semaphore_id
 *
 * @return void
 */
function codeHelperSemaphoreRelease($semaphore_id)
{
    // Liberando o Semáforo
    if (extension_loaded('sysvsem')) {
        sem_release($semaphore_id);
    }
}

/**
 * Limpa certos detalhes na String para evitar que de problema no Json e retorna o json como objeto ou array.
 *
 * @param string $json         String a ser aplicado o json_decode
 * @param bool   $is_array     Se true retorna como Array, se false como Objeto
 * @param bool   $remove_html  Se true vai remover o HTML do Json
 * @param bool   $return_error Se true e o Json der erro vai retornar o erro, caso contrário retorna false
 *
 * @return array|false|object|string
 */
function codeHelperJsonDecode(
    string $json,
    bool $is_array = true,
    bool $remove_html = false,
    bool $return_error =
    false
) {
    // Se a string JSON estiver vazia, retornar false
    if (empty(trim($json))) {
        return false;
    }

    if ($remove_html) {
        // Remover tags HTML e espaços extras no início/fim
        $json = trim(strip_tags($json));
    }

    // Remover BOM (Byte Order Mark) se presente
    $json = preg_replace('/^\xEF\xBB\xBF/', '', $json);

    // Remover caracteres de controle não imprimíveis, mas manter \n, \r e \t
    $json = preg_replace('/[\x00-\x08\x0B\x0C\x0E-\x1F\x7F]/', '', $json);

    // Substituir aspas simples por aspas duplas apenas em chaves JSON
    $json = preg_replace_callback(
        '/(?<!\\\\)\'(\w+)\'\s*:/',
        function ($matches) {
            return '"' . $matches[1] . '":';
        },
        $json
    );

    // Substituir vírgulas finais inválidas dentro de objetos/arrays JSON
    $json = preg_replace('/,\s*([\}\]])/', '$1', $json);

    // Tentar decodificar normalmente
    $data = json_decode($json, $is_array);

    // Se falhar, tentar remover espaços extras antes/depois de `:` e `,`
    if (json_last_error() !== JSON_ERROR_NONE) {
        $json = preg_replace('/\s*:\s*/', ':', $json);
        $json = preg_replace('/\s*,\s*/', ',', $json);
        $data = json_decode($json, $is_array);
    }

    // Se ainda houver erro, retornar `false`
    if (json_last_error() !== JSON_ERROR_NONE) {
        if ($return_error) {
            return 'Erro ao decodificar JSON: ' . json_last_error_msg();
        }

        return false;
    }

    // Retornando como Objeto
    if ($is_array === false) {
        return (object) $data;
    }

    // Retornando como Array
    return (array) $data;
}

/**
 * Retorna o tempo que falta com base no timeout do PHP ou o informado manualmente.
 *
 * @param int $max_execution_time
 *
 * @return int
 */
function codeHelperRemainingTimeout(int $max_execution_time = 0): int
{
    if (empty($max_execution_time)) {
        $max_execution_time = ini_get('max_execution_time');
    }
    $start_time = $_SERVER['REQUEST_TIME'];

    $current_time = time();
    $elapsed_time = $current_time - $start_time;

    return $max_execution_time - $elapsed_time;
}

/**
 * Formata um número com duas casas decimais e ponto, usado para converter principalmente um valor para centavos.
 *
 * @param float $number
 *
 * @return float
 */
function codeHelperNumberDecimal(float $number): float
{
    return (float) number_format(
        $number,
        2,
        '.',
        ''
    );
}

/**
 * Transforma um número em centavos, exemplo: R$ 50,80 vai virar 5080, logo 5080 centavos = R$ 50,80
 * Recurso usado por muitas APIs de Pagamento e ERP.
 *
 * @param float $number
 *
 * @return int
 */
function codeHelperNumberCent(float $number): int
{
    return (int) (codeHelperNumberDecimal($number) * 100);
}

/**
 * Transforma uma String em apenas números.
 *
 * @param string $string
 *
 * @return string
 */
function codeHelperOnlyNumbers(string $string): string
{
    return (string) preg_replace('/[^0-9]/', '', $string);
}

/**
 * Retorna o tempo agora, inclusive os Microssegundos, padrão 06:03:03.09
 * Primeiro hora, minuto, segundo com os microssegundos.
 *
 * @return string
 */
function codeHelperTimeNow(): string
{
    $microtime = microtime(true);
    $seconds = floor($microtime); // Parte inteira do timestamp
    $microseconds = round(($microtime - $seconds) * 100); // Microsegundos arredondados a duas casas decimais

    //return date('h', $seconds) . 'h ' . date('i', $seconds) . 'm ' . date('s', $seconds) . '.' . sprintf('%02d', $microseconds) . 's';
    return date('h', $seconds) . ':' . date('i', $seconds) . ':' . date('s', $seconds) . '.' . sprintf(
        '%02d',
        $microseconds
    );
}

/**
 * Cria o Slug para URL Amigável.
 *
 * @param string $text      Texto para transformar em Slug
 * @param array  $replace   Array a ser removido, ex: ['R$','&'], vai tirar o R$ e & do Slug
 * @param string $delimiter Delimitador para substituir o espaço, ex: -
 *
 * @return string
 */
function codeHelperSlug(string $text, array $replace = [], string $delimiter = '-'): string
{
    // Remove acentos e caracteres especiais
    $slug = iconv('UTF-8', 'ASCII//TRANSLIT', $text);

    // Substitui caracteres indesejados do array de replace
    if (!empty($replace)) {
        $slug = str_replace((array) $replace, '', $slug);
    }

    // Remove tudo que não for alfanumérico, espaços ou o delimitador escolhido
    $slug = preg_replace('/[^a-zA-Z0-9\s]/', '', $slug);

    // Substitui espaços por delimitador (padrão: "-")
    $slug = preg_replace('/\s+/', $delimiter, trim($slug));

    // Converte para minúsculas
    return strtolower($slug);
}

/**
 * Trata URL de Imagem para retornar no padrão correto, mesmo quando tem espaço ou algum outro padrão fora do comum
 * Exemplo: image/catalog/produtos/produto teste.png
 * Vira: image/catalog/produtos/produto%20teste.png
 * Colocando %20 no lugar do espaço, pois o Google e outros serviços podem ter problema com espaço direto na URL
 * O &amp; vira &
 * Já para URL recomendado o htmlspecialchars_decode($url); troca por exemplo &amp; por &.
 *
 * @param string $imageURL URL da imagem ou caminho parcial dela
 *
 * @return string
 */
function codeHelperFixImageURL(string $imageURL): string
{
    // Remove espaços em branco no início e no fim
    $imageURL = trim($imageURL);

    // Decodifica qualquer entidade HTML (corrige '&amp;' → '&')
    $imageURL = htmlspecialchars_decode($imageURL, ENT_QUOTES);

    // Se a URL já contém caracteres codificados, evita a dupla codificação
    if (preg_match('/%[0-9A-Fa-f]{2}/', $imageURL)) {
        $imageURL = urldecode($imageURL);
    }

    // Se a URL for absoluta, separa domínio e caminho
    if (preg_match('~^(https?://[^/]+)(/.*)$~', $imageURL, $matches)) {
        $domain = $matches[1] ?? ''; // Parte do domínio (https://seusite.com.br)
        $path = $matches[2] ?? ''; // Caminho da imagem (/image/catalog/...)

        // Garante que cada segmento do caminho esteja codificado corretamente (evita erro com `/`)
        $path = implode("/", array_map("rawurlencode", explode("/", $path)));
        $imageURL = $domain . $path;
    } else {
        // Garante que cada segmento do caminho esteja codificado corretamente (evita erro com `/`)
        $imageURL = implode("/", array_map("rawurlencode", explode("/", $imageURL)));
    }

    // Corrige casos onde `&` virou `%26`
    return str_replace('%26', '&', $imageURL);
}
