<?php

/**
 * @copyright Copyright © 2013-2025 Codemarket - Todos os direitos reservados
 * @author Felipo Antonoff <contato@codemarket.com.br>
 * @link https://www.codemarket.com.br
 * @link https://felipoantonoff.com
 * @date 08/02/2025 11:43
 **/

namespace Codemarket\System\Library\CodeApp;

/**
 * © Copyright 2013-2025 Codemarket - Todos os direitos reservados.
 * Sistema de Logs usando o Banco de Dados.
 * @package CodeApp
 */
class Log
{
    private $log;
    private $registry;
    private $db;
    private $session;
    private $table;
    private $update;
    private $status;

    /**
     * Construtor que chama o setup() para criar as tabelas e setar as variáveis.
     * @param $registry
     * @param $db
     * @param $table
     */
    public function __construct($registry, $db, $table = '')
    {
        /*
        $this->load->model('module/codemarket_module');
        $conf = $this->model_module_codemarket_module->getModulo('');
        */

        // Log no formato de arquivo para ele
        $this->log = new \log('Code-Log-' . date('d-m-Y') . '.log');
        $this->setup($registry, $db, $table);
    }

    /**
     * Setup - Cria a tabela code_log.
     *
     * @param $registry
     * @param $db
     * @param $table
     * @return void
     */
    private function setup($registry, $db, $table)
    {
        $this->registry = $registry;
        $this->db = $db;
        $this->session = $registry->get('session');

        // CONFIGURAÇÃO
        // Ativa ou desativa o recurso de Update, por padrão fica desativado
        // Se o foco for o desempenho, recomendado desativar para Loja com alta demanda
        $this->update = false;

        // Habilita/Desabilita a melhoria
        $this->status = true;

        $this->table = empty($table) ? DB_PREFIX . 'code_log' : DB_PREFIX . 'code_log_' . $table;

        // CRIANDO A TABELA CODE_LOG SE NÃO EXISTIR
        $this->db->query(
            "CREATE TABLE IF NOT EXISTS `" . $this->table . "` (
            `log_id` int(25) unsigned NOT NULL AUTO_INCREMENT COMMENT 'ID do Log',
            `level` enum('DEBUG','INFO','NOTICE','WARNING','ERROR','CRITICAL','ALERT','EMERGENCY') DEFAULT 'DEBUG' COMMENT 'Tipo de Log, Debug, Info, Notice e etc',
            `name` varchar(255) DEFAULT NULL COMMENT 'Identificar do Log, pode ser o nome de uma função',
            `group` varchar(255) DEFAULT NULL COMMENT 'Identificador do Log a nível maior, tipo o arquivo ou class',
            `file` text DEFAULT NULL COMMENT 'Nome ou caminho do arquivo',
            `line` int(6) DEFAULT NULL COMMENT 'Linha do Log',
            `message` longtext DEFAULT NULL COMMENT 'Mensagem do Log/conteúdo',
            `data` longtext DEFAULT NULL COMMENT 'Conteúdo extra do Log, normalmente Json',
            `quantity` int(11) DEFAULT NULL COMMENT 'Quantidade de vezes que repete o log, quando tem com os mesmos dados',
            `date_added` datetime DEFAULT NULL COMMENT 'Quando foi adicionado',
            `date_modified` datetime DEFAULT NULL COMMENT 'Quando foi editado (quantity)',
            PRIMARY KEY (`log_id`) USING BTREE,
            INDEX `level_index` (`level`),
            INDEX `name_index` (`name`),
            INDEX `line` (`line`),
            INDEX `quantity` (`quantity`),
            INDEX `date_added` (`date_added`),
            INDEX `date_modified` (`date_modified`)
          ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;
          "
        );
    }

    /**
     * Apenas um método de teste.
     * @return void
     */
    public function test()
    {
        //$query = $this->db->query("SELECT log_id FROM ".$this->table." LIMIT 1");
        //print_r($query);
        exit("Teste CodeLog OK");
    }

    /**
     * Recurso usado pelo Log nativo, sobrescrevendo o método write()
     * Usado quando troca o nativo pelo novo, mas mantém os mesmos Logs, sem o uso do add() diretto
     * Chama o método add() preenchendo os dados que faltam usando o backrace.
     *
     * @param string $message Mensagem do Log
     * @return false|int
     */
    public function write(string $message): int
    {
        if (!$this->status) {
            return 0;
        }

        $backtrace = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 2);
        //$this->log->write("debug_backtrace ".print_r($backtrace, true));
        $function = !empty($backtrace[1]['function']) ? $backtrace[1]['function'] : '';
        $line = !empty($backtrace[0]['line']) ? (int) $backtrace[0]['line'] : 0;
        $class = !empty($backtrace[1]['class']) ? $backtrace[1]['class'] : '';
        $file = !empty($backtrace[0]['file']) ? $backtrace[0]['file'] : '';
        $uuid = $this->session->data['codeAppUuidCall'];

        $function = 'LogDefault-' . $function;

        return $this->add('INFO', $function . ':' . $uuid, $class, $file, $line, $message, [], true);
    }

    /**
     * Adiciona os Logs.
     *
     * @param string $level 'DEBUG','INFO','NOTICE','WARNING','ERROR','CRITICAL','ALERT','EMERGENCY'
     * @param string $name Identificar do Log, pode ser o nome de uma função
     * @param string $group Identificador do Log a nível maior, tipo o arquivo ou class
     * @param string $file Arquivo
     * @param int $line Linha
     * @param string $message Mnesagem do Log
     * @param array $data Array contendo dados
     * @param bool $update Se true, vai atualizar na contagem se existir um log igual e se false cria um novo log
     * @return false|int
     */
    public function add(
        string $level,
        string $name,
        string $group,
        string $file,
        int $line,
        string $message,
        array $data,
        bool $update = true
    ): int {
        if (!$this->status) {
            return false;
        }

        // Verificando se está ativo o Update
        if ($update && $this->update) {
            $updateQuantity = $this->update($level, $name, $group, $file, $line, $message, $data);

            if (!empty($updateQuantity)) {
                $this->log->write('add() - Atualizado quantidade Log ID: ' . $updateQuantity);

                return $updateQuantity;
            }
        }

        // Adicionado o Log
        $this->db->query(
            "INSERT INTO " . $this->table . " 
            SET 
            `level` = '" . $this->db->escape($level) . "',
            `name` = '" . $this->db->escape($name) . "',
            `group` = '" . $this->db->escape($group) . "',
            `file` = '" . $this->db->escape($file) . "',
            `line` = '" . (int) $line . "',
            `message` = '" . $this->db->escape($message) . "',
            `data` = '" . $this->db->escape(json_encode($data)) . "',
            `quantity` = 1,
            `date_added` = NOW(),
            `date_modified` = NOW()
        "
        );

        $log_id = $this->db->getLastId();
        $this->log->write('add() - Adicionado o Log ID: ' . $log_id);

        return $log_id;
    }

    /**
     * Atualiza os Logs.
     *
     * @param string $level
     * @param string $name
     * @param string $group
     * @param string $file
     * @param int $line
     * @param string $message
     * @param array $data
     * @return false|int
     */
    private function update(
        string $level,
        string $name,
        string $group,
        string $file,
        int $line,
        string $message,
        array $data
    ) {
        if (!$this->status) {
            return false;
        }

        $this->log->write('update() - Rodado o Update()');

        $sql = "SELECT log_id, quantity FROM " . $this->table . "
        WHERE
        `level` = '" . $this->db->escape($level) . "' AND
        `name` = '" . $this->db->escape($name) . "' AND
        `group` = '" . $this->db->escape($group) . "' AND
        `file` = '" . $this->db->escape($file) . "' AND
        `line` = '" . (int) $line . "' AND
        `message` = '" . $this->db->escape($message) . "' AND
        `data` = '" . $this->db->escape(json_encode($data)) . "'
        LIMIT 1";

        // Verificando se já tem um Log semelhante
        $query = $this->db->query($sql)->row;

        if (empty($query['log_id'])) {
            return false;
        }

        // Sem atualizar
        // return 1;

        // Mudando a Quantidade e Data de Modificado
        $this->db->query(
            "
            UPDATE " . $this->table . "
            SET
            quantity = '" . (int) ($query['quantity'] + 1) . "',
            date_modified = NOW()
            WHERE
            log_id = '" . (int) $query['log_id'] . "'
        "
        );

        return $query['log_id'];
    }

    /**
     * Recurso em desenvolvimento, por enquanto só retorna a resposta da busca direto.
     *
     * @return false|void
     */
    public function get()
    {
        if (!$this->status) {
            return false;
        }

        $level = !empty($this->request->get['level']) ? trim($this->request->get['level']) : '';
        $name = !empty($this->request->get['name']) ? trim($this->request->get['name']) : '';
        $group = !empty($this->request->get['group']) ? trim($this->request->get['group']) : '';
        $file = !empty($this->request->get['file']) ? trim($this->request->get['file']) : '';
        $message = !empty($this->request->get['message']) ? trim($this->request->get['message']) : '';
        $data = !empty($this->request->get['data']) ? trim($this->request->get['data']) : '';
        $limit = !empty($this->request->get['limit']) ? trim($this->request->get['limit']) : 20;

        $sql = "SELECT * FROM " . $this->table . "
        WHERE
        `level` = '" . $this->db->escape($level) . "' OR
        `name` LIKE '%" . $this->db->escape($name) . "%' OR
        `group` LIKE '%" . $this->db->escape($group) . "%' OR
        `file` LIKE '%" . $this->db->escape($file) . "%' OR
        `message` LIKE '%" . $this->db->escape($message) . "%' OR
        `data` LIKE '%" . $this->db->escape($data) . "%'
        LIMIT $limit";

        $query = $this->db->query($sql)->rows;

        if (empty($query[0]['log_id'])) {
            return false;
        }

        foreach ($query as $key => $log) {
            if (empty($log['data'])) {
                continue;
            }
            $query[$key]['data'] = json_decode($log['data'], true);
            //echo $log['message'];
        }

        print_r($query);
    }

    /**
     * Limpa a tabela de Logs informada usando o TRUNCATE.
     *
     * @param string $table
     * @return bool|void
     */
    public function clean(string $table)
    {
        if (!$this->status) {
            return false;
        }

        $table = empty($table) ? DB_PREFIX . 'code_log' : DB_PREFIX . 'code_log_' . $table;
        echo "<h2>Limpando a tabela de Log: " . $table . "</h2>";

        $this->db->query("TRUNCATE TABLE `" . $table . "`");
        $this->log->write('clean() - Limpado os Logs da tabela: ' . $table);

        echo "<h2>Limpado os Logs com sucesso!</h2>";
    }
}
